from odoo import models, fields, api
from odoo.addons.loempia.util import html_sanitize_hyperlinks

class ModuleRating(models.Model):
    _name = 'loempia.module.rating'
    _order = 'id desc'
    _description = 'Apps Module Rating'

    def _get_default_author(self):
        return self.env.user.partner_id

    # copy the content of the mail message fields
    subject = fields.Char('Subject')
    body = fields.Html('Contents', default='', sanitize_style=True)
    rating = fields.Integer(string="Rating", default=5, required=True)
    date = fields.Datetime('Date', default=fields.Datetime.now)
    author_id = fields.Many2one(
        'res.partner', 'Author', index=True,
        ondelete='set null', default=_get_default_author,
        help="Author of the message. If not set, email_from may hold an email address that did not match any partner.")

    module_id = fields.Many2one('loempia.module', string='Reference Module', index=True, required=True, ondelete='cascade')
    template_id = fields.Many2one('loempia.module.template', string="Reference Template Module", index=True, required=True, ondelete='cascade')
    parent_id = fields.Many2one('loempia.module.rating', string='Parent message', help='Parent message used to make channels with multi-layers messages', ondelete='cascade')
    child_ids = fields.One2many('loempia.module.rating', 'parent_id', 'Child Messages')

    # moderation
    state = fields.Selection([
        ('unreviewed', 'Not reviewed'),
        ('pending', 'In review'),
        ('reviewed', 'Reviewed'),
    ], string='Status', required=True, readonly=True, default='unreviewed')
    report_count = fields.Integer(
        string="Number of reports",
        default=0,
        help="This represents the number of times the message was reported before being reviewed internally.")
    reviewer_id = fields.Many2one(
        'res.partner', 'Reviewer',
        ondelete='set null', help="Internal Employee who reviewed the message.")

    @api.model_create_multi
    def create(self, vals_list):
        for vals in vals_list:
            if 'body' in vals:
                vals['body'] = html_sanitize_hyperlinks(vals['body'])
        return super().create(vals_list)

    def write(self, values):
        if 'body' in values:
            values['body'] = html_sanitize_hyperlinks(values['body'])
        return super().write(values)

    def _module_notification(self, recipient, reply=False):
        """
            this function will send a notification mail to the specified recipient
            recipient: res.partner object
        """
        self.ensure_one()
        if not (recipient and recipient.notify_on_module_comment):
            return

        mail_template = self.env.ref('loempia.new_comment_on_module')

        email_context = self.env.context.copy()
        email_context.update({
            'receiver': recipient.name,
            'reply': reply,
        })
        render_ctx = dict(
            email_to=recipient.email,
            email_from="apps@odoo.com",
        )
        _ = mail_template.sudo().with_context(email_context).send_mail(res_id=self.module_id.id, raise_exception=False, email_values=render_ctx)

        return True

    def unlink(self):
        # As it will trigger writing on other models, we explicitly checked that the user
        # has the access on the rating and we then unlink the rating via sudo
        self.check_access_rights('unlink')
        self.check_access_rule('unlink')
        return super(ModuleRating, self.sudo()).unlink()

    def confirm_review(self):
        reviewer_id = self.env.user.partner_id
        self.write({
            'reviewer_id': reviewer_id.id,
            'state': 'reviewed'
        })

    def reset_review(self):
        if not self.env['res.users'].has_group('loempia.group_manager'):
            return True

        self.write({
            'reviewer_id': None,
            'state': 'unreviewed'
        })

    def website_url(self):
        self.ensure_one()
        url = '{module.page_url}/#loempia-comments'.format(module=self.module_id)
        return {
            'type': 'ir.actions.act_url',
            'url': url,
            'target': 'new',
        }

    def _report(self, reason, user, body):
        self.ensure_one()
        if self.state != 'unreviewed':
            return True

        mail_template = self.env.ref('loempia.rating_reported')

        email_context = self.env.context.copy()

        email_context.update({
            'name': self.module_id.shortdesc,
            'reporter': user.partner_id.name,
            'reason': reason,
            'body': body,
        })
        render_ctx = dict(
            email_to="apps@odoo.com",
            email_from=user.partner_id.email,
        )

        mail_id = mail_template.sudo().with_context(email_context).send_mail(res_id=self.id, raise_exception=False, email_values=render_ctx)

        _ = self.env['mail.mail'].browse(mail_id)
        self.write({
            'state': 'pending',
            'report_count': self.report_count + 1
        })

        return True
